using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Windows.Media.Media3D;
using System.Drawing;
using Firaxis.Utility;
using Firaxis.Granny;
using NexusBuddy.GrannyWrappers;
using NexusBuddy.GrannyInfos;
using NexusBuddy.FileOps;

namespace NexusBuddy
{ 

    public class CivNexusSixApplicationForm : Form
	{
		public static CivNexusSixApplicationForm form;
		public static IGrannyFile loadedFile;
        public static string applicationName = "CivNexus6";
        public static int major_version = 1;
        public static int minor_version = 0;
        public static int sub_minor_version = 0;
        
	    public string modelTemplateFilename;
	    public string sourceTemplatePath = "E:\\mod\\3D\\";
        
		private Random rand = new Random();
        private List<string> TempFiles = new List<string>();
		private IContainer components;
        private SplitContainer masterSplitContainer;
		private PropertyGrid properties;
		private ColumnHeader animation;
		private ColumnHeader duration;

		private ColumnHeader materialTypeColumnHeader;
        private ColumnHeader materialNameColumnHeader;
        private GroupBox fileInfoGroupBox;
        private RichTextBox fileInfoTextBox;
        private Label headerFilenameLabel;
        private SplitContainer leftHandSplitContainer;
        private TabControl mainTabControl;
        private TabPage grannyFileTabPage;
        private SplitContainer grannyFileSplitContainerA;
        private Panel mainButtonPanel;
        private Button openButton;
        private Button viewButton;
        private Button saveButton;
        private Button saveAsButton;
        private Button saveAnimationButton;
        private ListViewWithComboBox meshList;
        private ColumnHeader MeshName;
        private ColumnHeader Material;
        private SplitContainer grannyFileTabContainer;
        private ListView materialList;
        private ColumnHeader materialNameHeader;
        private ColumnHeader materialTypeHeader;
        private Panel materialButtonsPanel;
        private Button addMaterial;
        private Button deleteMaterial;
        private TabControl animationsTabControl;
        private TabPage grannyAnimsTabPage;
        private ListView animList;
        private ColumnHeader anim;
        private ColumnHeader timeSlice;
        private Panel panel1;
        private ToolTip viewButtonToolTip;
        private TabPage otherActionsTabPage;
        private Button exportNB2Button;
        private Button exportNB2CurrentModelButton;
        private Button br2ImportButton;
        private Button openFBXButton;
        private Button overwriteBR2Button;
        private Button overwriteMeshesButton;
        private TabPage selectModelTabPage;
        private ListView modelList;
        private ColumnHeader modelName;
        private ListViewItem lastModelListItemChecked;
        private Button resaveAllFilesInDirButton;
        private Int32 currentModelIndex;
        private RadioButton useSceneTemplateRadioButton;
        private RadioButton useLeaderTemplateRadioButton;
        private RadioButton useUnitTemplateRadioButton;
        private Label templateBR2OverwriteLabel;
        private Button removeAnimationsButton;
        private Button resaveAllFBXAsAnimsButton;
        private Button makeTemplateButton;
        private Label rescaleFactorLabel;
        private Button insertAdjustmentBoneButton;
        private TextBox rescaleFactorTextBox;
        private ComboBox bonesComboBox;
        private Label rescaleBoneNameLabel;
        private Button rescaleNamedBoneButton;
        private Button exportNA2Button;
        private Label endTimeTextBoxLabel;
        private Label startTimeTextBoxLabel;
        private Label classNameTextBoxLabel;
        private TextBox endTimeTextBox;
        private Label angleLabel;
        private Label axisLabel;
        private ComboBox axisComboBox;
        private ComboBox classNameComboBox;
        private TextBox angleTextBox;
        private Button concatenateNA2Button;
        private Label fpsLevel;
        private TextBox fpsTextBox;
        private Button openBR2Button;
        private Label nodeLabel;
        private TextBox nodeTextBox;
        private Button loadStringDatabaseButton;
        private TextBox startTimeTextBox;

		public CivNexusSixApplicationForm()
		{
			form = this;
			InitializeComponent();
            Available.Startup(applicationName);
            Context.Add(new GrannyContext());
			Context.Add(new ProjectConfig());
			Context.Get<GrannyContext>();
			Assembly executingAssembly = Assembly.GetExecutingAssembly();
            try
            {
                Stream manifestResourceStream = executingAssembly.GetManifestResourceStream("NexusBuddy.GrannyTemplates.model_template.fgx");
                modelTemplateFilename = Path.GetTempPath() + "modelTemplate.temp.fgx";
                FileStream fileStream = File.Create(modelTemplateFilename);
                ReadWriteStream(manifestResourceStream, fileStream);
                fileStream.Close();
            }
            catch (Exception)
            {
            }
		}

		private void ReadWriteStream(Stream readStream, Stream writeStream)
		{
			int num = 256;
			byte[] buffer = new byte[num];
			for (int i = readStream.Read(buffer, 0, num); i > 0; i = readStream.Read(buffer, 0, num))
			{
				writeStream.Write(buffer, 0, i);
			}
			readStream.Close();
			writeStream.Close();
		}

		protected override void OnClosing(CancelEventArgs e)
		{
            CleardownAllData();
            base.OnClosing(e);
		}

        private void CleardownAllData()
        {
            loadedFile = null;
            foreach (string current in TempFiles)
            {
                try
                {
                    File.Delete(current);
                }
                catch (Exception)
                {
                }
            }
        }

		private void AddAnimToListbox(IGrannyAnimation grannyAnimation)
		{
			animList.Items.Add(grannyAnimation.Name);
			animList.Items[animList.Items.Count - 1].SubItems.Add(grannyAnimation.Duration.ToString("f6", CultureInfo.InvariantCulture));
			animList.Items[animList.Items.Count - 1].SubItems.Add(grannyAnimation.TimeStep.ToString("f6", CultureInfo.InvariantCulture));
        }

		private void AddMeshToListbox(IGrannyMesh grannyMesh)
		{
			meshList.Items.Add(grannyMesh.Name);
			if (grannyMesh.MaterialBindings.Count.Equals(1))
			{
				meshList.Items[meshList.Items.Count - 1].SubItems.Add(grannyMesh.MaterialBindings[0].Name);
			}
			else
			{
				meshList.Items[meshList.Items.Count - 1].SubItems.Add("<unassigned>");
			}
			meshList.Items[meshList.Items.Count - 1].Tag = grannyMesh;
		}

        private void AddMaterialToListbox(IGrannyMaterial material)
		{
            materialList.Items.Add(material.Name);
            materialList.Items[materialList.Items.Count - 1].SubItems.Add(material.typeName);
            materialList.Items[materialList.Items.Count - 1].Tag = new MaterialProperties(material);
        }

        private void RefreshAppDataWithMessage(string message)
        {
            RefreshAppData();
            fileInfoTextBox.Text = fileInfoTextBox.Text + message;
        }

		private void RefreshAppData()
		{
            string fileInfo = "";
            if (loadedFile != null) {

                if (currentModelIndex == -1)
                {
                    currentModelIndex = 0;
                }
                
			    materialList.Items.Clear();
			    meshList.Items.Clear();
			    animList.Items.Clear();
			    properties.SelectedObject = null;

                if (loadedFile.Models.Count > 0 && loadedFile.Models[currentModelIndex].Skeleton != null)
                {
                    IGrannySkeleton skeleton = loadedFile.Models[currentModelIndex].Skeleton;
                    bonesComboBox.Items.Clear();
                    foreach (IGrannyBone bone in skeleton.Bones)
                    {
                        bonesComboBox.Items.Add(bone.Name);
                    }
                }

                if (modelList.Items.Count != loadedFile.Models.Count)
                {
                    foreach (IGrannyModel currentModel in loadedFile.Models)
                    {
                        modelList.Items.Add(currentModel.Name);
                    }
                    modelList.Items[currentModelIndex].Checked = true;
                }
                if (loadedFile.Models.Count > 0)
                {
                    foreach (IGrannyMesh currentMesh in loadedFile.Models[currentModelIndex].MeshBindings)
                    {
                        AddMeshToListbox(currentMesh);
                    }
                }
			    foreach (IGrannyMaterial currentMaterial in loadedFile.Materials)
			    {
                    AddMaterialToListbox(currentMaterial); 
			    }
                foreach (IGrannyAnimation currentAnimation in loadedFile.Animations)
			    {
                    AddAnimToListbox(currentAnimation);
			    }

                IGrannyFile file = loadedFile;
                string filename = Path.GetFileName(file.Filename);
                headerFilenameLabel.Text = filename;
                fileInfo += "Models: " + file.Models.Count + "     ";
                if (file.Models.Count > 0)
                {
                    fileInfo += "Current Model: " + file.Models[currentModelIndex].Name + " (Index:" + currentModelIndex + ")" + Environment.NewLine;
                    fileInfo += "Bones (Current Model): " + file.Models[currentModelIndex].Skeleton.Bones.Count + "     ";
                    fileInfo += "Meshes (Current Model): " + file.Models[currentModelIndex].MeshBindings.Count + "     ";
                }
                fileInfo += "Meshes (Total): " + file.Meshes.Count + Environment.NewLine;
                fileInfo += "Materials: " + file.Materials.Count + Environment.NewLine;
                fileInfo += "Animations: " + file.Animations.Count + Environment.NewLine;
            }
            fileInfoTextBox.Text = fileInfo;
		}

        private void RescaleButtonClick(object sender, EventArgs e)
        {
            GrannySkeletonWrapper skeletonWrapper = new GrannySkeletonWrapper(loadedFile.Models[currentModelIndex].Skeleton);
            GrannySkeletonInfo skeletonInfo = skeletonWrapper.readSkeletonInfo();

            foreach (GrannyBoneInfo curBoneInfo in skeletonInfo.bones)
            {
                curBoneInfo.parentIndex = curBoneInfo.parentIndex + 1;
            }

            float scaleFactor = 1f;
            try
            {
                scaleFactor = float.Parse(rescaleFactorTextBox.Text, CultureInfo.InvariantCulture);
            }
            catch (FormatException) {}

            Vector3D axisVector = new Vector3D(1.0d, 0.0d, 0.0d);
            if (axisComboBox.SelectedIndex == 1)
            {
                axisVector = new Vector3D(0.0d, 1.0d, 0.0d);
            }
            else if (axisComboBox.SelectedIndex == 2)
            {
                axisVector = new Vector3D(0.0d, 0.0d, 1.0d);
            }

            float angle = 1f;
            try
            {
                angle = float.Parse(angleTextBox.Text, CultureInfo.InvariantCulture);
            }
            catch (FormatException) { }
            
            Quaternion quat = new Quaternion(axisVector, angle);

            GrannyBoneInfo boneInfo = new GrannyBoneInfo();

            int j = 1;
            string adjBoneNameRoot = "CN6_ADJUSTMENT_BONE";
            foreach (IGrannyBone bone in skeletonWrapper.wrappedSkeleton.Bones)
            {
                if (bone.Name.StartsWith(adjBoneNameRoot))
                {
                    j++;
                }
            }

            boneInfo.name = adjBoneNameRoot + "_" + j;

            boneInfo.parentIndex = -1;
            GrannyTransformInfo transformInfo = new GrannyTransformInfo();
            float[] position = { 0f, 0f, 0f };
            float[] orientation = { (float)quat.X, (float)quat.Y, (float)quat.Z, (float)quat.W };
            float[] scaleShear = { scaleFactor, 0f, 0f, 0f, scaleFactor, 0f, 0f, 0f, scaleFactor };
            float[] invWorldTransform = { 1f, 0f, 0f, 0f, 0f, 1f, 0f, 0f, 0f, 0f, 1f, 0f, 0f, 0f, 0f, 1f };
            transformInfo.position = position;
            transformInfo.orientation = orientation;
            transformInfo.scaleShear = scaleShear;
            transformInfo.flags = 7;
            boneInfo.localTransform = transformInfo;
            boneInfo.inverseWorldTransform = invWorldTransform;
            skeletonInfo.bones.Insert(0, boneInfo);

            skeletonWrapper.writeSkeletonInfo(skeletonInfo);
            SaveAction();
            RefreshAppDataWithMessage("ADJUSTMENT BONE INSERTED.");
        }

        private void RescaleNamedBoneButtonClick(object sender, EventArgs e)
        {
            GrannySkeletonWrapper skeletonWrapper = new GrannySkeletonWrapper(loadedFile.Models[currentModelIndex].Skeleton);
            GrannySkeletonInfo skeletonInfo = skeletonWrapper.readSkeletonInfo();

            float scaleFactor = float.Parse(rescaleFactorTextBox.Text, CultureInfo.InvariantCulture);

            GrannyBoneInfo boneInfo = new GrannyBoneInfo();
            boneInfo.name = bonesComboBox.SelectedItem + "_FX";
            boneInfo.parentIndex = bonesComboBox.SelectedIndex;
            GrannyTransformInfo transformInfo = new GrannyTransformInfo();
            float[] position = { 0f, 0f, 0f };
            float[] orientation = { 0f, 0f, 0f, 1f };
            float[] scaleShear = { scaleFactor, 0f, 0f, 0f, scaleFactor, 0f, 0f, 0f, scaleFactor };
            float[] invWorldTransform = skeletonInfo.bones[bonesComboBox.SelectedIndex].inverseWorldTransform;
            transformInfo.position = position;
            transformInfo.orientation = orientation;
            transformInfo.scaleShear = scaleShear;
            transformInfo.flags = 4;
            boneInfo.localTransform = transformInfo;
            boneInfo.inverseWorldTransform = invWorldTransform;
            skeletonInfo.bones.Add(boneInfo);

            skeletonWrapper.writeSkeletonInfo(skeletonInfo);
            SaveAction();
            RefreshAppDataWithMessage("RESCALE BONE ADDED.");
        }

        private void ExportNA2ButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("Must have model file loaded to export animation.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            if (loadedFile != null)
            {
                OpenFileDialog openFileDialog = new OpenFileDialog();
                openFileDialog.Filter = "fgx files (*.fgx) | *.fgx";
                openFileDialog.FilterIndex = 0;
                openFileDialog.RestoreDirectory = true;
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    IGrannyFile animationFile = OpenFileAsTempFileCopy(openFileDialog.FileName, "tempanimopen");
                    if (animationFile.Animations.Count == 1)
                    {
                        int retryLimit = 5;
                        float startTime = -1;
                        try
                        {
                            startTime = float.Parse(startTimeTextBox.Text, CultureInfo.InvariantCulture);
                        }
                        catch (FormatException) { }

                        float endTime = -1;
                        try
                        {
                            endTime = float.Parse(endTimeTextBox.Text, CultureInfo.InvariantCulture);
                        }
                        catch (FormatException) { }

                        int fpsInput = 0;
                        try
                        {
                            fpsInput = int.Parse(fpsTextBox.Text, CultureInfo.InvariantCulture);
                        }
                        catch (FormatException) { }

                        int retryCount = NB2Exporter.exportNA2(loadedFile.Models[currentModelIndex], animationFile.Animations[0], openFileDialog.FileName, startTime, endTime, fpsInput, retryLimit);

                        if (retryCount == retryLimit)
                        {
                            RefreshAppDataWithMessage("ANIMATION EXPORT FAILED! Retry Count: " + retryCount);
                         }
                        else
                        {
                            RefreshAppDataWithMessage("ANIMATION EXPORTED! Retry Count: " + retryCount);
                        }             
                    }
                }
                
            }
        }

        private void OpenButtonClick(object sender, EventArgs e)
        {
            OpenButtonDialogAction("fgx/gr2 files (*.fgx)|*.fgx;*.gr2|br2 files (*.br2)|*.br2|All files (*.*)|*.*");
        }

        private void OpenFBXButtonClick(object sender, EventArgs e)
        {
            OpenButtonDialogAction("FBX files (*.fbx)|*.fbx|All files (*.*)|*.*");
        }

        private void OpenBR2ButtonClick(object sender, EventArgs e)
        {
            OpenButtonDialogAction("br2 files (*.br2)|*.br2|All files (*.*)|*.*");       
        }

        private void OpenButtonDialogAction(string filterString)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = filterString;
            openFileDialog.FilterIndex = 0;
            openFileDialog.RestoreDirectory = true;
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                string openFilename = openFileDialog.FileName;
                currentModelIndex = -1;
                modelList.Items.Clear();
                loadedFile = OpenFileAction(openFilename);
                RefreshAppDataWithMessage("FILE OPENED: " + openFilename);
            }
        }
        
        public IGrannyFile OpenFileAction(string openFilename)
        {
            IGrannyFile targetFile = OpenFileAsTempFileCopy(openFilename, "tempopen");
            bool saveFile = false;

            if (openFilename.ToLower().Contains(".fbx"))
            {
                openFilename = openFilename.Replace(".fbx", ".fgx");
                openFilename = openFilename.Replace(".FBX", ".fgx");
            }
            else if (openFilename.ToLower().Contains(".br2"))
            {
                openFilename = openFilename.Replace(".br2", ".fgx");
                openFilename = openFilename.Replace(".BR2", ".fgx");
            }
            else if (openFilename.ToLower().Contains(".gr2"))
            {
                openFilename = openFilename.Replace(".gr2", ".fgx");
                openFilename = openFilename.Replace(".GR2", ".fgx");
                saveFile = true;
            }

            targetFile.Filename = openFilename;

            if (saveFile)
            {
                targetFile.Save();
            }
        
            return targetFile;
        }

        public static string GetVersionString()
        {
            return major_version + "." + minor_version + "." + sub_minor_version;
        }

        public static void SetExporterInfo(GrannyFileWrapper fileWrapper)
        {
            fileWrapper.setExporterInfo(applicationName, major_version, minor_version, sub_minor_version, 0);
        }

        public IGrannyFile OpenFileAsTempFileCopy(string openFilename, string prefix)
        {
            GrannyContext grannyContext = Context.Get<GrannyContext>();
            string tempPath = Path.GetTempPath();
            string tempFilename = tempPath + prefix;
            if (openFilename.ToLower().Contains(".fbx") && !System.String.IsNullOrEmpty(nodeTextBox.Text))
            {
                string tempFilename2 = tempFilename;
                tempFilename2 = AppendRandomNumberAndFGXExtension(tempFilename2);

                FBXImporter.ImportFBXFile(openFilename, tempFilename2, nodeTextBox.Text);

                IGrannyFile fbxfgxfile = grannyContext.LoadGrannyFile(tempFilename2);
                //GrannyFileWrapper fileWrapper = new GrannyFileWrapper(fbxgr2file);
                //fileWrapper.setFromArtToolInfo("Blender", 2, 0);
                //float[] matrix = { 1f, 0f, 0f, 0f, 0f, 1f, 0f, -1f, 0f };
                //fileWrapper.setMatrix(matrix);
                //SetExporterInfo(fileWrapper);
                //fileWrapper.setFromFileName(openFilename);

                tempFilename = AppendRandomNumberAndFGXExtension(tempFilename);

                SaveAsAction(fbxfgxfile, tempFilename, false);

                TempFiles.Add(tempFilename);
                TempFiles.Add(tempFilename2);
            }
            else if (openFilename.ToLower().Contains(".br2"))
            {
                tempFilename = AppendRandomNumberAndFGXExtension(tempFilename);
                BR2Importer.importBR2(openFilename, tempFilename, grannyContext);
                TempFiles.Add(tempFilename);
            }
            else
            {
                tempFilename = AppendRandomNumberAndFGXExtension(tempFilename);
                TempFiles.Add(tempFilename);
                try
                {
                    if (File.Exists(tempFilename))
                    {
                        File.Delete(tempFilename);
                    }
                }
                catch (Exception)
                {
                }
                File.Copy(openFilename, tempFilename);
            }

            IGrannyFileLoader grannyFileLoader = new GrannyFileLoader();
            IGrannyFile targetFile = grannyFileLoader.LoadGrannyFile(tempFilename);
            return targetFile;
        }

       

        private string AppendRandomNumberAndFGXExtension(string tempFilename)
        {
            tempFilename += rand.Next(1000000).ToString();
            tempFilename += rand.Next(1000000).ToString();
            tempFilename += rand.Next(1000000).ToString();
            tempFilename += ".fgx";
            return tempFilename;
        }

        private void UpdateMatrix(object sender, EventArgs e)
        {
            GrannyFileWrapper fileWrapper = new GrannyFileWrapper(loadedFile);
            float[] matrix = { 1f, 0f, 0f, 0f, 0f, 1f, 0f, -1f, 0f };
            fileWrapper.setMatrix(matrix);
            SetExporterInfo(fileWrapper);
            SaveAction();
        }

        private void ViewButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("No file is currently loaded.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            ViewAction();
        }

        private void ViewAction()
        {
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.CreateNoWindow = false;
            startInfo.UseShellExecute = false;
            startInfo.FileName = "fgxviewer.exe";
            startInfo.WindowStyle = ProcessWindowStyle.Normal;
            if (loadedFile != null)
            {
                startInfo.Arguments = "\"" + loadedFile.Filename + "\"";
                Process.Start(startInfo);
            }
        }
		
		private void SaveAsButtonClick(object sender, EventArgs e)
		{
            if (loadedFile == null)
            {
                MessageBox.Show("No file is currently loaded.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            SaveFileDialog saveFileDialog = new SaveFileDialog();
			saveFileDialog.Filter = "fgx files (*.fgx)|*.fgx|All files (*.*)|*.*";
			saveFileDialog.FilterIndex = 0;
			saveFileDialog.RestoreDirectory = true;
			if (saveFileDialog.ShowDialog() == DialogResult.OK)
			{
				string fileName = saveFileDialog.FileName;
                SaveAsAction(loadedFile, fileName, true);       
			}
		}

        private void SaveButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("No file is currently loaded.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            DialogResult dialogResult = DialogResult.Yes;
            if (File.Exists(loadedFile.Filename))
            {
                dialogResult = MessageBox.Show("Are you sure you want to overwrite this file?", "Warning!", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
            }
            if (dialogResult == DialogResult.Yes)
            {
                SaveAction();
            }
        }

        private void SaveAction()
        {
            //GrannyFileWrapper fileWrapper = new GrannyFileWrapper(loadedFile);
            //float[] matrix = { 1f, 0f, 0f, 0f, 0f, 1f, 0f, -1f, 0f };
            //fileWrapper.setMatrix(matrix);
            string filename = loadedFile.Filename;
            SaveAsAction(loadedFile, filename, true);      
        }     

        public void SaveAsAction(IGrannyFile fileToSave, string fileName, bool updateAppDataAndSaveMessage)
        {
            GrannyFileWrapper fileWrapper = new GrannyFileWrapper(fileToSave);
            SetExporterInfo(fileWrapper);

            // Animation Fix for FBX sourced animations - not working!
            //if (fileToSave.Animations.Count > 0)
            //{
            //    foreach (IGrannyAnimation animation in fileToSave.Animations)
            //    {
            //        foreach(IGrannyTrackGroup trackGroup in animation.TrackGroups)
            //        {
            //            GrannyTrackGroupWrapper trackGroupWrapper = new GrannyTrackGroupWrapper(fileToSave.Animations[0].TrackGroups[0]);
            //            trackGroupWrapper.setAccumulationFlags(0);
            //        }
            //    }
            //}
            //fileWrapper.setArtToolPointerSize(64);
            //fileWrapper.setUnitsPerMeter(10f);

            string tempStagingFileName = fileName.Replace(".fgx", "_temp_" + rand.Next(1000000) + rand.Next(1000000) + ".fgx");
            fileToSave.Filename = tempStagingFileName;
            fileToSave.Source = "Tool";
            fileToSave.Save();
            if (File.Exists(fileName))
            {
                File.Delete(fileName);
            }
            File.Move(tempStagingFileName, fileName);
            fileToSave.Filename = fileName;
            
            GrannyContext grannyContext = Context.Get<GrannyContext>();
            string tempPath = Path.GetTempPath();
            string tempFileName = tempPath + "tempopen";
            tempFileName += rand.Next(1000000).ToString();
            tempFileName += rand.Next(1000000).ToString();
            tempFileName += ".fgx";
            TempFiles.Add(tempFileName);
            File.Copy(fileName, tempFileName);
            IGrannyFile savedFile = grannyContext.LoadGrannyFile(tempFileName);

            savedFile.Filename = fileName;
            savedFile.Source = "Tool";

            loadedFile = savedFile;
            modelList.Items.Clear();

            if (updateAppDataAndSaveMessage)
            {
                RefreshAppDataWithMessage("FILE SAVED: " + fileName);
            }
        }

        private string getShortFilenameForLoadedFile()
        {
            return Path.GetFileName(loadedFile.Filename);
        }

        private void SaveAnimationClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("No file is currently loaded.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = "fgx files (*.fgx)|*.fgx|All files (*.*)|*.*";
            saveFileDialog.FilterIndex = 0;
            saveFileDialog.RestoreDirectory = true;
            saveFileDialog.FileName = getShortFilenameForLoadedFile().Replace(".fgx", "_anim.fgx");
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                string fileName = saveFileDialog.FileName;
                SaveAnimationsAction(loadedFile, fileName);

                RefreshAppDataWithMessage("ANIMATION FILE SAVED.");
            }
        }

        private static List<IGrannyFile> SaveAnimationsAction(IGrannyFile grannyFile, string filename)
        {
            GrannyContext grannyContext = Context.Get<GrannyContext>();
            int i = 0;
            List<IGrannyFile> animationFiles = new List<IGrannyFile>();
            foreach (IGrannyAnimation animation in grannyFile.Animations)
            {
                string extension_string = "";
                if (i > 0)
                {
                    extension_string = i + "";
                }
                IGrannyFile animationFile = grannyContext.CreateEmptyGrannyFile(filename.Replace(".fgx", extension_string + ".fgx"));
                animationFile.AddAnimationReference(animation);
                animationFile.AddArtToolAndExporterReference(grannyFile);
                animationFile.Save();
                animationFiles.Add(animationFile);
                i++;
            }
            return animationFiles;
        }

        private void OverwriteMeshesButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("You need to open a file before you can overwrite meshes!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            GrannyContext grannyContext = Context.Get<GrannyContext>();
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "br2 files (*.br2)|*.br2";
            openFileDialog.FilterIndex = 0;
            openFileDialog.RestoreDirectory = true;

            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                string meshBR2filename = openFileDialog.FileName;
                BR2Importer.overwriteMeshes(loadedFile, meshBR2filename, grannyContext, currentModelIndex);
                RefreshAppDataWithMessage("MESHES OVERWRITTEN from " + meshBR2filename);
            }
        }

        private void RemoveAnimationsButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("No file is currently loaded.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            GrannyFileWrapper fileWrapper = new GrannyFileWrapper(loadedFile);
            fileWrapper.setNumAnimations(0);
            fileWrapper.setNumTrackGroups(0);
            SaveAction();
        }

        private void ConcatenateNA2ButtonClick(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog
            {
                SelectedPath = Settings.Default.RecentFolder
            };
            folderBrowserDialog.ShowNewFolderButton = false;
            folderBrowserDialog.RootFolder = Environment.SpecialFolder.Desktop;
            DialogResult result = folderBrowserDialog.ShowDialog();

            if (result == DialogResult.OK)
            {
                System.String tempFilename = "combined" + rand.Next(100000) + ".na2";
                Settings.Default.RecentFolder = folderBrowserDialog.SelectedPath;
                var files = Directory.GetFiles(folderBrowserDialog.SelectedPath, "*.*", SearchOption.TopDirectoryOnly)
                            .Where(s => s.ToLower().EndsWith(".na2"));

                TextWriter textWriter = new StreamWriter(tempFilename);
                textWriter.WriteLine("// Animation NA2 - Exported from " + applicationName);

                int frameSets = 0;
                foreach (string filename in files)
                {
                    if (filename.LastIndexOf("__") > 0)
                    {
                        frameSets++;
                    }
                }

                textWriter.WriteLine("FrameSets: " + frameSets);

                System.String outputFilename = null;

                foreach (string filename in files)
                {
                    if (filename.LastIndexOf("__") > 0 ) { 
                        outputFilename = filename.Substring(0, filename.LastIndexOf("__")) + ".na2";
                        StreamReader fileReader = new StreamReader(filename);
                        while (fileReader.Peek() >= 0)
                        {
                            string fileLine = fileReader.ReadLine();
                            if (!fileLine.StartsWith("//") && !fileLine.StartsWith("FrameSets"))
                            {
                                textWriter.Write(fileLine + "\n");
                            }
                        }
                        fileReader.Close();
                    }
                }
                textWriter.Close();

                if (File.Exists(outputFilename))
                {
                    File.Delete(outputFilename);
                }
                    if (File.Exists(tempFilename))
                {
                    File.Copy(tempFilename, outputFilename);
                    File.Delete(tempFilename);
                }
                RefreshAppDataWithMessage("NA2 FILES CONCATENATED.");
            }
        }

        private void ResaveAllGR2FilesInDirAsAnimsClick(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog
            {
                SelectedPath = Settings.Default.RecentFolder
            };
            folderBrowserDialog.ShowNewFolderButton = false;
            folderBrowserDialog.RootFolder = Environment.SpecialFolder.Desktop;
            DialogResult result = folderBrowserDialog.ShowDialog();

            string outputDirectory = "fgx_anm_output";

            if (result == DialogResult.OK)
            {
                Settings.Default.RecentFolder = folderBrowserDialog.SelectedPath;
                var files = Directory.GetFiles(folderBrowserDialog.SelectedPath, "*.*", SearchOption.TopDirectoryOnly)
                            .Where(s => s.ToLower().EndsWith(".gr2"));

                Directory.CreateDirectory(folderBrowserDialog.SelectedPath + "\\" + outputDirectory);

                foreach (string filename in files)
                {
                    IGrannyFile grannyFile = OpenFileAction(filename);
                    string savefilename = filename;
                    if (savefilename.ToLower().Contains(".gr2"))
                    {
                        savefilename = savefilename.Replace(".gr2", ".fgx");
                        savefilename = savefilename.Replace(".GR2", ".fgx");
                    }
                    string shortSavefilename = Path.GetFileName(savefilename);
                    List<IGrannyFile> animationFiles = SaveAnimationsAction(grannyFile, folderBrowserDialog.SelectedPath + "\\"  + outputDirectory + "\\" + shortSavefilename);
                    foreach (IGrannyFile animationFile in animationFiles)
                    {
                        MetadataWriter.WriteGeoAnimFile(animationFile, 0, classNameComboBox.Text);
                    }     
                }
                CleardownAllData();
                RefreshAppDataWithMessage("ALL GR2 FILES IN DIRECTORY SAVED AS .fgx/.anm");
            }
        }

        private void WriteGeoFileButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("You need to open a file before you can write .geo/.anm file!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            MetadataWriter.WriteGeoAnimFile(loadedFile, currentModelIndex, classNameComboBox.Text);
            RefreshAppDataWithMessage("WRITE .geo/.anm FILE COMPLETE!");
        }

        private void WriteGeometrySetDataClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("You need to open a file before you can write GeometrySet data!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }
            MetadataWriter.WriteGeometrySet(loadedFile);
            RefreshAppDataWithMessage("WRITE GeometrySet XML DATA COMPLETE!");
        }

        private void DeleteMaterialClick(object sender, EventArgs e)
		{
			List<ListViewItem> list = new List<ListViewItem>();
			if (materialList.SelectedItems.Count > 0 && MessageBox.Show("Are you sure you want to delete the selected materials? There is no undo.", "Warning!", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.Yes)
			{
				foreach (object selectedMaterialItem in materialList.SelectedItems)
				{
					MaterialProperties materialProperties = (selectedMaterialItem as ListViewItem).Tag as MaterialProperties;
					foreach (IGrannyMesh mesh in loadedFile.Meshes)
					{
						mesh.RemoveMaterialBinding(materialProperties.GetMaterial());
					}

					loadedFile.RemoveMaterial(materialProperties.GetMaterial());
					loadedFile.Materials.Remove(materialProperties.GetMaterial());
					list.Add(selectedMaterialItem as ListViewItem);

					foreach (object meshListItem in meshList.Items)
					{
						if ((meshListItem as ListViewItem).SubItems[1].Text == materialProperties.Name)
						{
							(meshListItem as ListViewItem).SubItems[1].Text = "<unassigned>";
						}
					}
				}
				foreach (ListViewItem materialListItem in list)
				{
					materialList.Items.Remove(materialListItem);
				}
				properties.SelectedObject = null;
			}
            RefreshAppData();
		}

		private void AddMaterialClick(object sender, EventArgs e)
		{
            if (loadedFile == null)
            {
                return;
            }

            List<string> materialNames = new List<string>();
            foreach (IGrannyMaterial currentMaterial in loadedFile.Materials)
            {
                materialNames.Add(currentMaterial.Name);
            }

            string newMaterialName = "Default_Material";
            for (int i=1; i<1000000; i++)
            {
                string tryMaterialName = "Default_Material_" + i;
                if (!materialNames.Contains(tryMaterialName))
                {
                    newMaterialName = tryMaterialName;
                    break;
                }
            }

            IGrannyMaterial materialFromTemplateFile = GetMaterialFromTemplateFile();
            materialFromTemplateFile.Name = newMaterialName;
            loadedFile.AddMaterialReference(materialFromTemplateFile);
            AddMaterialToListbox(materialFromTemplateFile);
            RefreshAppData();
        }

        public IGrannyMaterial GetMaterialFromTemplateFile()
        {
        	if (loadedFile == null)
        	{
        		return null;
        	}
        	GrannyContext grannyContext = Context.Get<GrannyContext>();
        	string tempPath = Path.GetTempPath();
        	string text = tempPath + "temptemplate";
        	text += rand.Next(10000000).ToString();
        	text += rand.Next(10000000).ToString();
        	text += rand.Next(10000000).ToString();
        	text += rand.Next(10000000).ToString();
        	text += ".fgx";
        	TempFiles.Add(text);

            File.Copy(modelTemplateFilename, text);
        	IGrannyFile grannyFile = grannyContext.LoadGrannyFile(text);
        	foreach (IGrannyMaterial material in grannyFile.Materials)
        	{
                if (material.Name == "Default_Material")
                {
                    return material;
                }
        	}
        	return null;
        }

        private void ExportNB2ButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("You need to open a file before you can export to NB2!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            NB2Exporter.exportAllModelsToNB2(loadedFile);
            RefreshAppDataWithMessage("EXPORT TO NB2 COMPLETE.");
        }

        private void ExportNB2CurrentModelButtonClick(object sender, EventArgs e)
        {
            if (loadedFile == null)
            {
                MessageBox.Show("You need to open a file before you can export to NB2!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
                return;
            }

            NB2Exporter.exportNB2(loadedFile, currentModelIndex);
            RefreshAppDataWithMessage("EXPORT TO NB2 COMPLETE.");
        }

        //Make Model Template
        private void MakeTemplateButtonClick(object sender, EventArgs e)
        {
            string filename = sourceTemplatePath + "Hoplite_ArmorA.fgx";

            IGrannyFile file = form.OpenFileAsTempFileCopy(filename, "tempimport");

            GrannyMeshWrapper meshWrapper = new GrannyMeshWrapper(file.Meshes[0]);
            meshWrapper.setName("BLANK_MESH");
            meshWrapper.setNumVertices(1);
            meshWrapper.setNumIndices16(1);
            meshWrapper.setGroup0TriCount(1);
            meshWrapper.setNumMaterialBindings(1);

            foreach(IGrannyMaterial material in file.Meshes[0].MaterialBindings)
            {
                if (material.Name == "Hoplite_Material")
                {
                    GrannyMaterialWrapper materialWrapper = new GrannyMaterialWrapper(material);
                    materialWrapper.setName("Default_Material");
                }
            }

            GrannyModelWrapper modelWrapper = new GrannyModelWrapper(file.Models[0]);
            GrannySkeletonWrapper skeletonWrapper = new GrannySkeletonWrapper(modelWrapper.wrappedModel.Skeleton);

            modelWrapper.setNumMeshBindings(1);
            modelWrapper.setName("BLANK_MODEL");

            skeletonWrapper.setNumBones(1);
            skeletonWrapper.setName("BLANK_SKELETON");

            GrannyFileWrapper fileWrapper = new GrannyFileWrapper(file);
            fileWrapper.setNumMeshes(1);
            fileWrapper.setNumVertexDatas(1);
            fileWrapper.setNumSkeletons(1);
            fileWrapper.setNumTriTopologies(1);
            //fileWrapper.setNumMaterials(1);
            fileWrapper.setNumTextures(1);

            //file.Meshes.Clear();

            SaveAsAction(file, sourceTemplatePath + "model_template.fgx", true);
        }

		public void UpdateMaterialBinding(string material)
		{
			if (meshList.SelectedItems.Count == 0 || meshList.SelectedItems.Count > 1)
			{
				return;
			}
			IGrannyMesh grannyMesh = meshList.SelectedItems[0].Tag as IGrannyMesh;
			IGrannyMaterial[] array = grannyMesh.MaterialBindings.ToArray();
			for (int i = 0; i < array.Length; i++)
			{
				IGrannyMaterial kMaterial = array[i];
				grannyMesh.RemoveMaterialBinding(kMaterial);
			}
			grannyMesh.MaterialBindings.Clear();
			if (material == "<unassigned>")
			{
				return;
			}
			foreach (IGrannyMaterial current in loadedFile.Materials)
			{
				if (current.Name == material)
				{
					grannyMesh.AddMaterialBinding(current);
                    break;
				}
			}
		}

		private void PropertiesPropertyValueChanged(object s, PropertyValueChangedEventArgs e)
		{
			if (e.ChangedItem.Label == "Name" && properties.SelectedObject is MaterialProperties)
			{
				bool flag = true;
				foreach (ListViewItem listViewItem in materialList.Items)
				{
					if (listViewItem.Text == (string)e.ChangedItem.Value && properties.SelectedObject as MaterialProperties != listViewItem.Tag as MaterialProperties)
					{
						(properties.SelectedObject as MaterialProperties).Name = (e.OldValue as string);
						MessageBox.Show("A material of this name already exists.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Hand);
						flag = false;
					}
				}
				if (flag)
				{
					foreach (ListViewItem listViewItem2 in materialList.Items)
					{
						if (listViewItem2.Text == (string)e.OldValue)
						{
							listViewItem2.Text = (e.ChangedItem.Value as string);
						}
					}
					foreach (ListViewItem listViewItem3 in meshList.Items)
					{
						if (listViewItem3.SubItems[1].Text == (string)e.OldValue)
						{
							listViewItem3.SubItems[1].Text = (e.ChangedItem.Value as string);
						}
					}
				}
			}
		}

		private void MeshListSelectedIndexChanged(object sender, EventArgs e)
		{
			List<object> list = new List<object>();
			if (meshList.SelectedItems.Count > 0)
			{
				foreach (object current in meshList.SelectedItems)
				{
					IGrannyMesh grannyMesh = (current as ListViewItem).Tag as IGrannyMesh;
					if (grannyMesh != null)
					{
						list.Add(grannyMesh);
					}
				}
				properties.SelectedObjects = list.ToArray();
			}
		}
        
        private void MaterialListSelectedIndexChanged(object sender, EventArgs e)
        {
            List<object> list = new List<object>();
            if (materialList.SelectedItems.Count > 0)
            {
                foreach (object current in materialList.SelectedItems)
                {
                    MaterialProperties indieMaterial = (current as ListViewItem).Tag as MaterialProperties;
                    if (indieMaterial != null)
                    {
                        list.Add(indieMaterial);
                    }
                    
                }
                properties.SelectedObjects = list.ToArray();
            }
        }

		protected override void Dispose(bool disposing)
		{
			if (disposing && components != null)
			{
				components.Dispose();
			}
			base.Dispose(disposing);
		}

        private void ModelListCheckChanged(object sender, ItemCheckEventArgs e)
        {
            int preCurrentModelIndex = currentModelIndex;

            // if we have the lastItem set as checked, and it is different
            // item than the one that fired the event, uncheck it
            if (lastModelListItemChecked != null && lastModelListItemChecked.Checked
                && lastModelListItemChecked != modelList.Items[e.Index])
            {
                // uncheck the last item and store the new one
                lastModelListItemChecked.Checked = false;
            }

            // store current item
            lastModelListItemChecked = modelList.Items[e.Index];

            currentModelIndex = lastModelListItemChecked.Index;

            if (preCurrentModelIndex != currentModelIndex)
            {
                RefreshAppData();
            }
        }

        private void InitializeComponent()
		{
            components = new Container();
            ComponentResourceManager resources = new ComponentResourceManager(typeof(CivNexusSixApplicationForm));
            masterSplitContainer = new SplitContainer();
            leftHandSplitContainer = new SplitContainer();
            mainButtonPanel = new Panel();
            openFBXButton = new Button();
            overwriteBR2Button = new Button();
            openButton = new Button();
            saveButton = new Button();
            saveAsButton = new Button();
            saveAnimationButton = new Button();
            mainTabControl = new TabControl();
            grannyFileTabPage = new TabPage();
            grannyFileSplitContainerA = new SplitContainer();
            meshList = new ListViewWithComboBox();
            MeshName = new ColumnHeader();
            Material = new ColumnHeader();
            grannyFileTabContainer = new SplitContainer();
            materialList = new ListView();
            materialNameHeader = new ColumnHeader();
            materialTypeHeader = new ColumnHeader();
            materialButtonsPanel = new Panel();
            addMaterial = new Button();
            deleteMaterial = new Button();
            animationsTabControl = new TabControl();
            grannyAnimsTabPage = new TabPage();
            animList = new ListView();
            anim = new ColumnHeader();
            timeSlice = new ColumnHeader();
            otherActionsTabPage = new TabPage();
            nodeLabel = new Label();
            nodeTextBox = new TextBox();
            openBR2Button = new Button();
            fpsLevel = new Label();
            fpsTextBox = new TextBox();
            concatenateNA2Button = new Button();
            angleLabel = new Label();
            axisLabel = new Label();
            axisComboBox = new ComboBox();
            classNameComboBox = new ComboBox();
            endTimeTextBoxLabel = new Label();
            startTimeTextBoxLabel = new Label();
            classNameTextBoxLabel = new Label();
            endTimeTextBox = new TextBox();
            startTimeTextBox = new TextBox();
            exportNA2Button = new Button();
            rescaleBoneNameLabel = new Label();
            rescaleNamedBoneButton = new Button();
            bonesComboBox = new ComboBox();
            rescaleFactorLabel = new Label();
            insertAdjustmentBoneButton = new Button();
            angleTextBox = new TextBox();
            rescaleFactorTextBox = new TextBox();
            makeTemplateButton = new Button();
            removeAnimationsButton = new Button();
            resaveAllFBXAsAnimsButton = new Button();
            templateBR2OverwriteLabel = new Label();
            useSceneTemplateRadioButton = new RadioButton();
            useLeaderTemplateRadioButton = new RadioButton();
            useUnitTemplateRadioButton = new RadioButton();
            resaveAllFilesInDirButton = new Button();
            exportNB2CurrentModelButton = new Button();
            overwriteMeshesButton = new Button();
            exportNB2Button = new Button();
            selectModelTabPage = new TabPage();
            modelList = new ListView();
            modelName = new ColumnHeader();
            panel1 = new Panel();
            headerFilenameLabel = new Label();
            viewButton = new Button();
            fileInfoGroupBox = new GroupBox();
            fileInfoTextBox = new RichTextBox();
            properties = new PropertyGrid();
            animation = new ColumnHeader();
            duration = new ColumnHeader();
            materialTypeColumnHeader = new ColumnHeader();
            materialNameColumnHeader = new ColumnHeader();
            viewButtonToolTip = new ToolTip(components);
            br2ImportButton = new Button();
            loadStringDatabaseButton = new Button();
            masterSplitContainer.BeginInit();
            masterSplitContainer.Panel1.SuspendLayout();
            masterSplitContainer.Panel2.SuspendLayout();
            masterSplitContainer.SuspendLayout();
            leftHandSplitContainer.BeginInit();
            leftHandSplitContainer.Panel1.SuspendLayout();
            leftHandSplitContainer.Panel2.SuspendLayout();
            leftHandSplitContainer.SuspendLayout();
            mainButtonPanel.SuspendLayout();
            mainTabControl.SuspendLayout();
            grannyFileTabPage.SuspendLayout();
            grannyFileSplitContainerA.BeginInit();
            grannyFileSplitContainerA.Panel1.SuspendLayout();
            grannyFileSplitContainerA.Panel2.SuspendLayout();
            grannyFileSplitContainerA.SuspendLayout();
            grannyFileTabContainer.BeginInit();
            grannyFileTabContainer.Panel1.SuspendLayout();
            grannyFileTabContainer.Panel2.SuspendLayout();
            grannyFileTabContainer.SuspendLayout();
            materialButtonsPanel.SuspendLayout();
            animationsTabControl.SuspendLayout();
            grannyAnimsTabPage.SuspendLayout();
            otherActionsTabPage.SuspendLayout();
            selectModelTabPage.SuspendLayout();
            panel1.SuspendLayout();
            fileInfoGroupBox.SuspendLayout();
            SuspendLayout();
            // 
            // masterSplitContainer
            // 
            masterSplitContainer.Dock = DockStyle.Fill;
            masterSplitContainer.Location = new Point(0, 0);
            masterSplitContainer.Name = "masterSplitContainer";
            // 
            // masterSplitContainer.Panel1
            // 
            masterSplitContainer.Panel1.Controls.Add(leftHandSplitContainer);
            // 
            // masterSplitContainer.Panel2
            // 
            masterSplitContainer.Panel2.Controls.Add(panel1);
            masterSplitContainer.Panel2.Controls.Add(fileInfoGroupBox);
            masterSplitContainer.Panel2.Controls.Add(properties);
            masterSplitContainer.Size = new Size(1008, 618);
            masterSplitContainer.SplitterDistance = 492;
            masterSplitContainer.TabIndex = 1;
            // 
            // leftHandSplitContainer
            // 
            leftHandSplitContainer.Dock = DockStyle.Fill;
            leftHandSplitContainer.Location = new Point(0, 0);
            leftHandSplitContainer.Name = "leftHandSplitContainer";
            leftHandSplitContainer.Orientation = Orientation.Horizontal;
            // 
            // leftHandSplitContainer.Panel1
            // 
            leftHandSplitContainer.Panel1.Controls.Add(mainButtonPanel);
            // 
            // leftHandSplitContainer.Panel2
            // 
            leftHandSplitContainer.Panel2.Controls.Add(mainTabControl);
            leftHandSplitContainer.Size = new Size(492, 618);
            leftHandSplitContainer.SplitterDistance = 42;
            leftHandSplitContainer.TabIndex = 2;
            // 
            // mainButtonPanel
            // 
            mainButtonPanel.Controls.Add(openFBXButton);
            mainButtonPanel.Controls.Add(overwriteBR2Button);
            mainButtonPanel.Controls.Add(openButton);
            mainButtonPanel.Controls.Add(saveButton);
            mainButtonPanel.Controls.Add(saveAsButton);
            mainButtonPanel.Controls.Add(saveAnimationButton);
            mainButtonPanel.Dock = DockStyle.Fill;
            mainButtonPanel.Location = new Point(0, 0);
            mainButtonPanel.Name = "mainButtonPanel";
            mainButtonPanel.Size = new Size(492, 42);
            mainButtonPanel.TabIndex = 9;
            // 
            // openFBXButton
            // 
            openFBXButton.Location = new Point(155, 5);
            openFBXButton.Name = "openFBXButton";
            openFBXButton.Size = new Size(87, 33);
            openFBXButton.TabIndex = 7;
            openFBXButton.Text = "Open BR2";
            openFBXButton.UseVisualStyleBackColor = true;
            openFBXButton.Click += OpenBR2ButtonClick;
            // 
            // overwriteBR2Button
            // 
            overwriteBR2Button.Location = new Point(67, 5);
            overwriteBR2Button.Name = "overwriteBR2Button";
            overwriteBR2Button.Size = new Size(82, 33);
            overwriteBR2Button.TabIndex = 6;
            overwriteBR2Button.Text = "Ovwr. BR2";
            overwriteBR2Button.UseVisualStyleBackColor = true;
            overwriteBR2Button.Click += OverwriteMeshesButtonClick;
            // 
            // openButton
            // 
            openButton.Location = new Point(4, 5);
            openButton.Name = "openButton";
            openButton.Size = new Size(57, 33);
            openButton.TabIndex = 2;
            openButton.Text = "Open";
            openButton.UseVisualStyleBackColor = true;
            openButton.Click += OpenButtonClick;
            // 
            // saveButton
            // 
            saveButton.Location = new Point(248, 5);
            saveButton.Name = "saveButton";
            saveButton.Size = new Size(66, 33);
            saveButton.TabIndex = 3;
            saveButton.Text = "Save";
            saveButton.UseVisualStyleBackColor = true;
            saveButton.Click += SaveButtonClick;
            // 
            // saveAsButton
            // 
            saveAsButton.Location = new Point(320, 5);
            saveAsButton.Name = "saveAsButton";
            saveAsButton.Size = new Size(70, 33);
            saveAsButton.TabIndex = 4;
            saveAsButton.Text = "Save As";
            saveAsButton.UseVisualStyleBackColor = true;
            saveAsButton.Click += SaveAsButtonClick;
            // 
            // saveAnimationButton
            // 
            saveAnimationButton.Location = new Point(396, 5);
            saveAnimationButton.Name = "saveAnimationButton";
            saveAnimationButton.Size = new Size(90, 33);
            saveAnimationButton.TabIndex = 5;
            saveAnimationButton.Text = "Save Anim.";
            saveAnimationButton.UseVisualStyleBackColor = true;
            saveAnimationButton.Click += SaveAnimationClick;
            // 
            // mainTabControl
            // 
            mainTabControl.Controls.Add(grannyFileTabPage);
            mainTabControl.Controls.Add(otherActionsTabPage);
            mainTabControl.Controls.Add(selectModelTabPage);
            mainTabControl.Dock = DockStyle.Fill;
            mainTabControl.Location = new Point(0, 0);
            mainTabControl.Name = "mainTabControl";
            mainTabControl.SelectedIndex = 0;
            mainTabControl.Size = new Size(492, 572);
            mainTabControl.TabIndex = 2;
            // 
            // grannyFileTabPage
            // 
            grannyFileTabPage.Controls.Add(grannyFileSplitContainerA);
            grannyFileTabPage.Location = new Point(4, 25);
            grannyFileTabPage.Name = "grannyFileTabPage";
            grannyFileTabPage.Padding = new Padding(3);
            grannyFileTabPage.Size = new Size(484, 543);
            grannyFileTabPage.TabIndex = 0;
            grannyFileTabPage.Text = "Edit Model";
            grannyFileTabPage.UseVisualStyleBackColor = true;
            // 
            // grannyFileSplitContainerA
            // 
            grannyFileSplitContainerA.Dock = DockStyle.Left;
            grannyFileSplitContainerA.Location = new Point(3, 3);
            grannyFileSplitContainerA.Name = "grannyFileSplitContainerA";
            grannyFileSplitContainerA.Orientation = Orientation.Horizontal;
            // 
            // grannyFileSplitContainerA.Panel1
            // 
            grannyFileSplitContainerA.Panel1.Controls.Add(meshList);
            // 
            // grannyFileSplitContainerA.Panel2
            // 
            grannyFileSplitContainerA.Panel2.Controls.Add(grannyFileTabContainer);
            grannyFileSplitContainerA.Size = new Size(480, 537);
            grannyFileSplitContainerA.SplitterDistance = 195;
            grannyFileSplitContainerA.TabIndex = 0;
            // 
            // meshList
            // 
            meshList.Columns.AddRange(new ColumnHeader[] {MeshName,Material});
            meshList.Dock = DockStyle.Fill;
            meshList.FullRowSelect = true;
            meshList.GridLines = true;
            meshList.Location = new Point(0, 0);
            meshList.MultiSelect = false;
            meshList.Name = "meshList";
            meshList.Size = new Size(480, 195);
            meshList.TabIndex = 2;
            meshList.UseCompatibleStateImageBehavior = false;
            meshList.View = View.Details;
            meshList.SelectedIndexChanged += MeshListSelectedIndexChanged;
            // 
            // MeshName
            // 
            MeshName.Tag = "Mesh Name";
            MeshName.Text = "Mesh Name";
            MeshName.Width = 180;
            // 
            // Material
            // 
            Material.Text = "Material";
            Material.Width = 200;
            // 
            // grannyFileTabContainer
            // 
            grannyFileTabContainer.Dock = DockStyle.Fill;
            grannyFileTabContainer.Location = new Point(0, 0);
            grannyFileTabContainer.Name = "grannyFileTabContainer";
            grannyFileTabContainer.Orientation = Orientation.Horizontal;
            // 
            // grannyFileTabContainer.Panel1
            // 
            grannyFileTabContainer.Panel1.Controls.Add(materialList);
            grannyFileTabContainer.Panel1.Controls.Add(materialButtonsPanel);
            // 
            // grannyFileTabContainer.Panel2
            // 
            grannyFileTabContainer.Panel2.Controls.Add(animationsTabControl);
            grannyFileTabContainer.Size = new Size(480, 338);
            grannyFileTabContainer.SplitterDistance = 162;
            grannyFileTabContainer.TabIndex = 5;
            // 
            // materialList
            // 
            materialList.Columns.AddRange(new ColumnHeader[] {materialNameHeader, materialTypeHeader});
            materialList.Dock = DockStyle.Fill;
            materialList.FullRowSelect = true;
            materialList.GridLines = true;
            materialList.Location = new Point(0, 0);
            materialList.Name = "materialList";
            materialList.Size = new Size(480, 131);
            materialList.TabIndex = 6;
            materialList.UseCompatibleStateImageBehavior = false;
            materialList.View = View.Details;
            materialList.SelectedIndexChanged += MaterialListSelectedIndexChanged;
            // 
            // materialNameHeader
            // 
            materialNameHeader.Text = "Material";
            materialNameHeader.Width = 180;
            // 
            // materialTypeHeader
            // 
            materialTypeHeader.Text = "Type";
            materialTypeHeader.Width = 200;
            // 
            // materialButtonsPanel
            // 
            materialButtonsPanel.Controls.Add(addMaterial);
            materialButtonsPanel.Controls.Add(deleteMaterial);
            materialButtonsPanel.Dock = DockStyle.Bottom;
            materialButtonsPanel.Location = new Point(0, 131);
            materialButtonsPanel.Name = "materialButtonsPanel";
            materialButtonsPanel.Size = new Size(480, 31);
            materialButtonsPanel.TabIndex = 7;
            // 
            // addMaterial
            // 
            addMaterial.Location = new Point(12, 5);
            addMaterial.Name = "addMaterial";
            addMaterial.Size = new Size(75, 23);
            addMaterial.TabIndex = 1;
            addMaterial.Text = "Add";
            addMaterial.UseVisualStyleBackColor = true;
            addMaterial.Click += AddMaterialClick;
            // 
            // deleteMaterial
            // 
            deleteMaterial.Location = new Point(95, 5);
            deleteMaterial.Name = "deleteMaterial";
            deleteMaterial.Size = new Size(75, 23);
            deleteMaterial.TabIndex = 0;
            deleteMaterial.Text = "Delete";
            deleteMaterial.UseVisualStyleBackColor = true;
            deleteMaterial.Click += DeleteMaterialClick;
            // 
            // animationsTabControl
            // 
            animationsTabControl.Controls.Add(grannyAnimsTabPage);
            animationsTabControl.Dock = DockStyle.Fill;
            animationsTabControl.Location = new Point(0, 0);
            animationsTabControl.Name = "animationsTabControl";
            animationsTabControl.SelectedIndex = 0;
            animationsTabControl.Size = new Size(480, 172);
            animationsTabControl.TabIndex = 1;
            // 
            // grannyAnimsTabPage
            // 
            grannyAnimsTabPage.Controls.Add(animList);
            grannyAnimsTabPage.Location = new Point(4, 25);
            grannyAnimsTabPage.Name = "grannyAnimsTabPage";
            grannyAnimsTabPage.Padding = new Padding(3);
            grannyAnimsTabPage.Size = new Size(472, 143);
            grannyAnimsTabPage.TabIndex = 0;
            grannyAnimsTabPage.Text = "Granny Anims";
            grannyAnimsTabPage.UseVisualStyleBackColor = true;
            // 
            // animList
            // 
            animList.Columns.AddRange(new ColumnHeader[] {anim, timeSlice});
            animList.Dock = DockStyle.Fill;
            animList.FullRowSelect = true;
            animList.GridLines = true;
            animList.Location = new Point(3, 3);
            animList.Name = "animList";
            animList.Size = new Size(466, 137);
            animList.TabIndex = 10;
            animList.UseCompatibleStateImageBehavior = false;
            animList.View = View.Details;
            // 
            // anim
            // 
            anim.Text = "Animation";
            anim.Width = 100;
            // 
            // timeSlice
            // 
            timeSlice.Text = "Time Slice";
            timeSlice.Width = 98;
            // 
            // otherActionsTabPage
            // 
            otherActionsTabPage.Controls.Add(nodeLabel);
            otherActionsTabPage.Controls.Add(nodeTextBox);
            otherActionsTabPage.Controls.Add(openBR2Button);
            otherActionsTabPage.Controls.Add(fpsLevel);
            otherActionsTabPage.Controls.Add(fpsTextBox);
            otherActionsTabPage.Controls.Add(concatenateNA2Button);
            otherActionsTabPage.Controls.Add(angleLabel);
            otherActionsTabPage.Controls.Add(axisLabel);
            otherActionsTabPage.Controls.Add(axisComboBox);
            otherActionsTabPage.Controls.Add(classNameComboBox);
            otherActionsTabPage.Controls.Add(endTimeTextBoxLabel);
            otherActionsTabPage.Controls.Add(startTimeTextBoxLabel);
            otherActionsTabPage.Controls.Add(classNameTextBoxLabel);
            otherActionsTabPage.Controls.Add(endTimeTextBox);
            otherActionsTabPage.Controls.Add(startTimeTextBox);
            otherActionsTabPage.Controls.Add(exportNA2Button);
            otherActionsTabPage.Controls.Add(rescaleBoneNameLabel);
            otherActionsTabPage.Controls.Add(rescaleNamedBoneButton);
            otherActionsTabPage.Controls.Add(bonesComboBox);
            otherActionsTabPage.Controls.Add(rescaleFactorLabel);
            otherActionsTabPage.Controls.Add(insertAdjustmentBoneButton);
            otherActionsTabPage.Controls.Add(angleTextBox);
            otherActionsTabPage.Controls.Add(rescaleFactorTextBox);
            otherActionsTabPage.Controls.Add(makeTemplateButton);
            otherActionsTabPage.Controls.Add(removeAnimationsButton);
            otherActionsTabPage.Controls.Add(resaveAllFBXAsAnimsButton);
            otherActionsTabPage.Controls.Add(templateBR2OverwriteLabel);
            otherActionsTabPage.Controls.Add(useSceneTemplateRadioButton);
            otherActionsTabPage.Controls.Add(useLeaderTemplateRadioButton);
            otherActionsTabPage.Controls.Add(useUnitTemplateRadioButton);
            otherActionsTabPage.Controls.Add(resaveAllFilesInDirButton);
            otherActionsTabPage.Controls.Add(loadStringDatabaseButton);
            otherActionsTabPage.Controls.Add(exportNB2CurrentModelButton);
            otherActionsTabPage.Controls.Add(overwriteMeshesButton);
            otherActionsTabPage.Controls.Add(exportNB2Button);
            otherActionsTabPage.Location = new Point(4, 25);
            otherActionsTabPage.Name = "otherActionsTabPage";
            otherActionsTabPage.Padding = new Padding(3);
            otherActionsTabPage.Size = new Size(484, 543);
            otherActionsTabPage.TabIndex = 1;
            otherActionsTabPage.Text = "Additional Actions";
            otherActionsTabPage.UseVisualStyleBackColor = true;
            // 
            // nodeLabel
            // 
            nodeLabel.AutoSize = true;
            nodeLabel.Location = new Point(6, 328);
            nodeLabel.Name = "nodeLabel";
            nodeLabel.Size = new Size(42, 17);
            nodeLabel.TabIndex = 47;
            nodeLabel.Text = "Node";
            // 
            // nodeTextBox
            // 
            nodeTextBox.Location = new Point(52, 325);
            nodeTextBox.Name = "nodeTextBox";
            nodeTextBox.Size = new Size(204, 22);
            nodeTextBox.TabIndex = 46;
            // 
            // openBR2Button
            // 
            openBR2Button.Location = new Point(6, 274);
            openBR2Button.Name = "openBR2Button";
            openBR2Button.Size = new Size(250, 40);
            openBR2Button.TabIndex = 45;
            openBR2Button.Text = "Open FBX";
            openBR2Button.UseVisualStyleBackColor = true;
            openBR2Button.Click += OpenFBXButtonClick;
            // 
            // fpsLevel
            // 
            fpsLevel.AutoSize = true;
            fpsLevel.Location = new Point(13, 197);
            fpsLevel.Name = "fpsLevel";
            fpsLevel.Size = new Size(93, 17);
            fpsLevel.TabIndex = 44;
            fpsLevel.Text = "FPS Override";
            // 
            // fpsTextBox
            // 
            fpsTextBox.ForeColor = SystemColors.ActiveCaptionText;
            fpsTextBox.Location = new Point(117, 194);
            fpsTextBox.Name = "fpsTextBox";
            fpsTextBox.Size = new Size(139, 22);
            fpsTextBox.TabIndex = 43;
            fpsTextBox.Text = "60";
            // 
            // concatenateNA2Button
            // 
            concatenateNA2Button.Location = new Point(6, 228);
            concatenateNA2Button.Name = "concatenateNA2Button";
            concatenateNA2Button.Size = new Size(250, 40);
            concatenateNA2Button.TabIndex = 42;
            concatenateNA2Button.Text = "Concatenate All NA2 Files in Directory";
            concatenateNA2Button.UseVisualStyleBackColor = true;
            concatenateNA2Button.Click += ConcatenateNA2ButtonClick;
            // 
            // angleLabel
            // 
            angleLabel.AutoSize = true;
            angleLabel.Location = new Point(260, 387);
            angleLabel.Name = "angleLabel";
            angleLabel.Size = new Size(101, 17);
            angleLabel.TabIndex = 41;
            angleLabel.Text = "Rotation Angle";
            // 
            // axisLabel
            // 
            axisLabel.AutoSize = true;
            axisLabel.Location = new Point(260, 357);
            axisLabel.Name = "axisLabel";
            axisLabel.Size = new Size(33, 17);
            axisLabel.TabIndex = 41;
            axisLabel.Text = "Axis";
            // 
            // axisComboBox
            // 
            axisComboBox.FormattingEnabled = true;
            axisComboBox.Items.AddRange(new object[] {
            "X (Left-Right)",
            "Y (Forward-Back)",
            "Z (Up-Down)"});
            axisComboBox.Location = new Point(316, 354);
            axisComboBox.Name = "axisComboBox";
            axisComboBox.Size = new Size(159, 24);
            axisComboBox.TabIndex = 40;
            axisComboBox.Text = "X (Left-Right)";

            // 
            // classNameTextBoxLabel
            // 
            classNameTextBoxLabel.AutoSize = true;
            classNameTextBoxLabel.Location = new Point(270, 106);
            classNameTextBoxLabel.Name = "classNameTextBoxLabel";
            classNameTextBoxLabel.Size = new Size(99, 17);
            classNameTextBoxLabel.TabIndex = 48;
            classNameTextBoxLabel.Text = "Class";
            classNameTextBoxLabel.TextAlign = ContentAlignment.MiddleRight;

            // 
            // classNameComboBox
            // 
            classNameComboBox.FormattingEnabled = true;
            classNameComboBox.Items.AddRange(new object[] {
            "DecalGeometry",
            "LandmarkModel",
            "LandmarkObstructionProfile",
            "Leader",
            "Leader_ShadowVolume",
            "UILensModel",
            "Unit",
            "VFXModel",
            "WonderMovieCamera"
            });
            classNameComboBox.Location = new Point(316, 102);
            classNameComboBox.Name = "classNameComboBox";
            classNameComboBox.Size = new Size(159, 24);
            classNameComboBox.TabIndex = 47;
            classNameComboBox.Text = "Unit";

            // 
            // endTImeTextBoxLabel
            // 
            endTimeTextBoxLabel.AutoSize = true;
            endTimeTextBoxLabel.Location = new Point(13, 170);
            endTimeTextBoxLabel.Name = "endTimeTextBoxLabel";
            endTimeTextBoxLabel.Size = new Size(99, 17);
            endTimeTextBoxLabel.TabIndex = 39;
            endTimeTextBoxLabel.Text = "NA2 End Time";
            endTimeTextBoxLabel.TextAlign = ContentAlignment.MiddleRight;
            // 
            // startTimeTextBoxLabel
            // 
            startTimeTextBoxLabel.AutoSize = true;
            startTimeTextBoxLabel.Location = new Point(13, 143);
            startTimeTextBoxLabel.Name = "startTimeTextBoxLabel";
            startTimeTextBoxLabel.Size = new Size(104, 17);
            startTimeTextBoxLabel.TabIndex = 38;
            startTimeTextBoxLabel.Text = "NA2 Start Time";
            startTimeTextBoxLabel.TextAlign = ContentAlignment.MiddleRight;
            // 
            // endTimeTextBox
            // 
            endTimeTextBox.Location = new Point(117, 167);
            endTimeTextBox.Name = "endTimeTextBox";
            endTimeTextBox.Size = new Size(139, 22);
            endTimeTextBox.TabIndex = 37;
            // 
            // startTimeTextBox
            // 
            startTimeTextBox.Location = new Point(117, 140);
            startTimeTextBox.Name = "startTimeTextBox";
            startTimeTextBox.Size = new Size(139, 22);
            startTimeTextBox.TabIndex = 37;
            // 
            // exportNA2Button
            // 
            exportNA2Button.Location = new Point(6, 94);
            exportNA2Button.Name = "exportNA2Button";
            exportNA2Button.Size = new Size(250, 40);
            exportNA2Button.TabIndex = 36;
            exportNA2Button.Text = "Export Animation to NA2";
            exportNA2Button.UseVisualStyleBackColor = true;
            exportNA2Button.Click += ExportNA2ButtonClick;
            // 
            // rescaleBoneNameLabel
            // 
            rescaleBoneNameLabel.AutoSize = true;
            rescaleBoneNameLabel.Location = new Point(174, 505);
            rescaleBoneNameLabel.Name = "rescaleBoneNameLabel";
            rescaleBoneNameLabel.Size = new Size(87, 17);
            rescaleBoneNameLabel.TabIndex = 35;
            rescaleBoneNameLabel.Text = "Parent Bone";
            // 
            // rescaleNamedBoneButton
            // 
            rescaleNamedBoneButton.Location = new Point(262, 458);
            rescaleNamedBoneButton.Name = "rescaleNamedBoneButton";
            rescaleNamedBoneButton.Size = new Size(213, 38);
            rescaleNamedBoneButton.TabIndex = 34;
            rescaleNamedBoneButton.Text = "Add Scaled FX Bone and Save";
            rescaleNamedBoneButton.UseVisualStyleBackColor = true;
            rescaleNamedBoneButton.Click += RescaleNamedBoneButtonClick;
            // 
            // bonesComboBox
            // 
            bonesComboBox.FormattingEnabled = true;
            bonesComboBox.Location = new Point(262, 502);
            bonesComboBox.Name = "bonesComboBox";
            bonesComboBox.Size = new Size(213, 24);
            bonesComboBox.TabIndex = 33;
            // 
            // rescaleFactorLabel
            // 
            rescaleFactorLabel.AutoSize = true;
            rescaleFactorLabel.Location = new Point(260, 328);
            rescaleFactorLabel.Name = "rescaleFactorLabel";
            rescaleFactorLabel.Size = new Size(103, 17);
            rescaleFactorLabel.TabIndex = 32;
            rescaleFactorLabel.Text = "Rescale Factor";
            // 
            // insertAdjustmentBoneButton
            // 
            insertAdjustmentBoneButton.Location = new Point(262, 274);
            insertAdjustmentBoneButton.Name = "insertAdjustmentBoneButton";
            insertAdjustmentBoneButton.Size = new Size(213, 40);
            insertAdjustmentBoneButton.TabIndex = 31;
            insertAdjustmentBoneButton.Text = "Insert Adjustment Bone and Save";
            insertAdjustmentBoneButton.UseVisualStyleBackColor = true;
            insertAdjustmentBoneButton.Click += RescaleButtonClick;
            // 
            // angleTextBox
            // 
            angleTextBox.Location = new Point(366, 384);
            angleTextBox.Name = "angleTextBox";
            angleTextBox.Size = new Size(109, 22);
            angleTextBox.TabIndex = 30;
            angleTextBox.Text = "0";
            // 
            // rescaleFactorTextBox
            // 
            rescaleFactorTextBox.Location = new Point(366, 325);
            rescaleFactorTextBox.Name = "rescaleFactorTextBox";
            rescaleFactorTextBox.Size = new Size(109, 22);
            rescaleFactorTextBox.TabIndex = 30;
            rescaleFactorTextBox.Text = "1";
            // 
            // makeTemplateButton
            // 
            makeTemplateButton.Location = new Point(262, 186);
            makeTemplateButton.Name = "makeTemplateButton";
            makeTemplateButton.Size = new Size(213, 38);
            makeTemplateButton.TabIndex = 29;
            makeTemplateButton.Text = "Make Template";
            makeTemplateButton.UseVisualStyleBackColor = true;
            makeTemplateButton.Click += MakeTemplateButtonClick;
            makeTemplateButton.Hide();
   
            // 
            // removeAnimationsButton
            // 
            removeAnimationsButton.Location = new Point(262, 186);
            removeAnimationsButton.Name = "removeAnimationsButton";
            removeAnimationsButton.Size = new Size(213, 40);
            removeAnimationsButton.TabIndex = 28;
            removeAnimationsButton.Text = "Remove Animations and Save";
            removeAnimationsButton.UseVisualStyleBackColor = true;
            removeAnimationsButton.Click += RemoveAnimationsButtonClick; //UpdateMatrix;
            // 
            // resaveAllFBXAsAnimsButton
            // 
            resaveAllFBXAsAnimsButton.Location = new Point(262, 140);
            resaveAllFBXAsAnimsButton.Name = "resaveAllFBXAsAnimsButton";
            resaveAllFBXAsAnimsButton.Size = new Size(213, 40);
            resaveAllFBXAsAnimsButton.TabIndex = 27;
            resaveAllFBXAsAnimsButton.Text = "Convert all .gr2 Files in Directory to .fgx/.anm";
            resaveAllFBXAsAnimsButton.UseVisualStyleBackColor = true;
            resaveAllFBXAsAnimsButton.Click += ResaveAllGR2FilesInDirAsAnimsClick;
            //resaveAllFBXAsAnimsButton.Hide();
            // 
            // templateBR2OverwriteLabel
            // 
            templateBR2OverwriteLabel.AutoSize = true;
            templateBR2OverwriteLabel.Font = new Font("Microsoft Sans Serif", 7.3125F, FontStyle.Bold, GraphicsUnit.Point, 0);
            templateBR2OverwriteLabel.Location = new Point(8, 374);
            templateBR2OverwriteLabel.Name = "templateBR2OverwriteLabel";
            templateBR2OverwriteLabel.Size = new Size(198, 16);
            templateBR2OverwriteLabel.TabIndex = 24;
            templateBR2OverwriteLabel.Text = "Template for BR2 Overwrite";
            templateBR2OverwriteLabel.Hide();
            // 
            // useSceneTemplateRadioButton
            // 
            useSceneTemplateRadioButton.AutoSize = true;
            useSceneTemplateRadioButton.Location = new Point(29, 452);
            useSceneTemplateRadioButton.Name = "useSceneTemplateRadioButton";
            useSceneTemplateRadioButton.Size = new Size(161, 21);
            useSceneTemplateRadioButton.TabIndex = 23;
            useSceneTemplateRadioButton.TabStop = true;
            useSceneTemplateRadioButton.Text = "Use Scene Template";
            useSceneTemplateRadioButton.UseVisualStyleBackColor = true;
            useSceneTemplateRadioButton.Hide();
            // 
            // useLeaderTemplateRadioButton
            // 
            useLeaderTemplateRadioButton.AutoSize = true;
            useLeaderTemplateRadioButton.Location = new Point(29, 426);
            useLeaderTemplateRadioButton.Name = "useLeaderTemplateRadioButton";
            useLeaderTemplateRadioButton.Size = new Size(166, 21);
            useLeaderTemplateRadioButton.TabIndex = 22;
            useLeaderTemplateRadioButton.TabStop = true;
            useLeaderTemplateRadioButton.Text = "Use Leader Template";
            useLeaderTemplateRadioButton.UseVisualStyleBackColor = true;
            useLeaderTemplateRadioButton.Hide();
            // 
            // useUnitTemplateRadioButton
            // 
            useUnitTemplateRadioButton.AutoSize = true;
            useUnitTemplateRadioButton.Checked = true;
            useUnitTemplateRadioButton.Location = new Point(29, 400);
            useUnitTemplateRadioButton.Name = "useUnitTemplateRadioButton";
            useUnitTemplateRadioButton.Size = new Size(146, 21);
            useUnitTemplateRadioButton.TabIndex = 21;
            useUnitTemplateRadioButton.TabStop = true;
            useUnitTemplateRadioButton.Text = "Use Unit Template";
            useUnitTemplateRadioButton.UseVisualStyleBackColor = true;
            useUnitTemplateRadioButton.Hide();
            // 
            // resaveAllFilesInDirButton
            // 
            resaveAllFilesInDirButton.Location = new Point(262, 50);
            resaveAllFilesInDirButton.Name = "resaveAllFilesInDirButton";
            resaveAllFilesInDirButton.Size = new Size(213, 40);
            resaveAllFilesInDirButton.TabIndex = 18;
            resaveAllFilesInDirButton.Text = "Create Geometry/Animation (.geo/.anm) File";
            resaveAllFilesInDirButton.UseVisualStyleBackColor = true;
            resaveAllFilesInDirButton.Click += WriteGeoFileButtonClick;
            // 
            // exportNB2CurrentModelButton
            // 
            exportNB2CurrentModelButton.Location = new Point(6, 50);
            exportNB2CurrentModelButton.Name = "exportNB2CurrentModelButton";
            exportNB2CurrentModelButton.Size = new Size(250, 40);
            exportNB2CurrentModelButton.TabIndex = 15;
            exportNB2CurrentModelButton.Text = "Export to NB2 (Current Model)";
            exportNB2CurrentModelButton.UseVisualStyleBackColor = true;
            exportNB2CurrentModelButton.Click += ExportNB2CurrentModelButtonClick;
            // 
            // overwriteMeshesButton
            // 
            overwriteMeshesButton.Location = new Point(6, 364);
            overwriteMeshesButton.Name = "overwriteMeshesButton";
            overwriteMeshesButton.Size = new Size(250, 40);
            overwriteMeshesButton.TabIndex = 14;
            overwriteMeshesButton.Text = "Overwrite Meshes From BR2 (Current Model)";
            overwriteMeshesButton.UseVisualStyleBackColor = true;
            overwriteMeshesButton.Click += OverwriteMeshesButtonClick;
            // 
            // exportNB2Button
            // 
            exportNB2Button.Location = new Point(6, 6);
            exportNB2Button.Name = "exportNB2Button";
            exportNB2Button.Size = new Size(250, 40);
            exportNB2Button.TabIndex = 11;
            exportNB2Button.Text = "Export to NB2 (All Models)";
            exportNB2Button.UseVisualStyleBackColor = true;
            exportNB2Button.Click += ExportNB2ButtonClick;       
            // 
            // selectModelTabPage
            // 
            selectModelTabPage.Controls.Add(modelList);
            selectModelTabPage.Location = new Point(4, 25);
            selectModelTabPage.Name = "selectModelTabPage";
            selectModelTabPage.Size = new Size(484, 543);
            selectModelTabPage.TabIndex = 2;
            selectModelTabPage.Text = "Select Model";
            selectModelTabPage.UseVisualStyleBackColor = true;
            // 
            // modelList
            // 
            modelList.CheckBoxes = true;
            modelList.Columns.AddRange(new ColumnHeader[] {modelName});
            modelList.Dock = DockStyle.Fill;
            modelList.FullRowSelect = true;
            modelList.GridLines = true;
            modelList.HeaderStyle = ColumnHeaderStyle.Nonclickable;
            modelList.Location = new Point(0, 0);
            modelList.MultiSelect = false;
            modelList.Name = "modelList";
            modelList.Size = new Size(484, 543);
            modelList.TabIndex = 0;
            modelList.UseCompatibleStateImageBehavior = false;
            modelList.View = View.Details;
            modelList.ItemCheck += ModelListCheckChanged;
            // 
            // modelName
            // 
            modelName.Text = "Model Name";
            modelName.Width = 300;
            // 
            // panel1
            // 
            panel1.Controls.Add(headerFilenameLabel);
            panel1.Controls.Add(viewButton);
            panel1.Dock = DockStyle.Top;
            panel1.Location = new Point(0, 0);
            panel1.Name = "panel1";
            panel1.Size = new Size(512, 42);
            panel1.TabIndex = 8;
            // 
            // headerFilenameLabel
            // 
            headerFilenameLabel.AutoSize = true;
            headerFilenameLabel.Font = new Font("Microsoft Sans Serif", 10F, FontStyle.Bold, GraphicsUnit.Point, 0);
            headerFilenameLabel.Location = new Point(3, 12);
            headerFilenameLabel.Name = "headerFilenameLabel";
            headerFilenameLabel.Size = new Size(110, 20);
            headerFilenameLabel.TabIndex = 3;
            headerFilenameLabel.Text = "No file open";
            // 
            // viewButton
            // 
            viewButton.Location = new Point(412, 3);
            viewButton.Name = "viewButton";
            viewButton.Size = new Size(97, 35);
            viewButton.TabIndex = 6;
            viewButton.Text = "View";
            viewButtonToolTip.SetToolTip(viewButton, "View .fbx in Granny Viewer");
            viewButton.UseVisualStyleBackColor = true;
            viewButton.Click += ViewButtonClick;
            // 
            // fileInfoGroupBox
            // 
            fileInfoGroupBox.Controls.Add(fileInfoTextBox);
            fileInfoGroupBox.Dock = DockStyle.Bottom;
            fileInfoGroupBox.Location = new Point(0, 46);
            fileInfoGroupBox.Name = "fileInfoGroupBox";
            fileInfoGroupBox.Size = new Size(512, 159);
            fileInfoGroupBox.TabIndex = 2;
            fileInfoGroupBox.TabStop = false;
            fileInfoGroupBox.Text = "Current File Info";
            // 
            // fileInfoTextBox
            // 
            fileInfoTextBox.Dock = DockStyle.Fill;
            fileInfoTextBox.Enabled = false;
            fileInfoTextBox.Location = new Point(3, 18);
            fileInfoTextBox.Name = "fileInfoTextBox";
            fileInfoTextBox.Size = new Size(506, 138);
            fileInfoTextBox.TabIndex = 0;
            fileInfoTextBox.Text = "";
            // 
            // properties
            // 
            properties.Dock = DockStyle.Bottom;
            properties.Location = new Point(0, 205);
            properties.Name = "properties";
            properties.Size = new Size(512, 413);
            properties.TabIndex = 0;
            properties.PropertyValueChanged += PropertiesPropertyValueChanged;
            
            // 
            // animation
            // 
            animation.Tag = "";
            animation.Text = "Animation";
            animation.Width = 149;
            // 
            // duration
            // 
            duration.Text = "Duration";
            duration.Width = 94;
           
            // 
            // materialTypeColumnHeader
            // 
            materialTypeColumnHeader.Text = "Material Type";
            materialTypeColumnHeader.Width = 138;
            // 
            // materialNameColumnHeader
            // 
            materialNameColumnHeader.Tag = "";
            materialNameColumnHeader.Text = "Material Name";
            materialNameColumnHeader.Width = 149;
            // 
            // br2ImportButton
            // 
            br2ImportButton.Location = new Point(0, 0);
            br2ImportButton.Name = "br2ImportButton";
            br2ImportButton.Size = new Size(75, 23);
            br2ImportButton.TabIndex = 0;
            // 
            // loadStringDatabaseButton
            // 
            loadStringDatabaseButton.Location = new Point(262, 6);
            loadStringDatabaseButton.Name = "loadStringDatabaseButton";
            loadStringDatabaseButton.Size = new Size(213, 40);
            loadStringDatabaseButton.TabIndex = 17;
            loadStringDatabaseButton.Text = "Create GeometrySet .xml for .ast file";
            loadStringDatabaseButton.UseVisualStyleBackColor = true;
            loadStringDatabaseButton.Click += WriteGeometrySetDataClick;
            // 
            // CivNexusSixApplicationForm
            // 
            ClientSize = new Size(1008, 618);
            Controls.Add(masterSplitContainer);
            Icon = (Icon)resources.GetObject("$this.Icon");
            Name = "CivNexusSixApplicationForm";
            Text = applicationName + " " + GetVersionString() + " - 3D Graphics Tool for Civilization 6";
            masterSplitContainer.Panel1.ResumeLayout(false);
            masterSplitContainer.Panel2.ResumeLayout(false);
            ((ISupportInitialize)masterSplitContainer).EndInit();
            masterSplitContainer.ResumeLayout(false);
            leftHandSplitContainer.Panel1.ResumeLayout(false);
            leftHandSplitContainer.Panel2.ResumeLayout(false);
            ((ISupportInitialize)leftHandSplitContainer).EndInit();
            leftHandSplitContainer.ResumeLayout(false);
            mainButtonPanel.ResumeLayout(false);
            mainTabControl.ResumeLayout(false);
            grannyFileTabPage.ResumeLayout(false);
            grannyFileSplitContainerA.Panel1.ResumeLayout(false);
            grannyFileSplitContainerA.Panel2.ResumeLayout(false);
            ((ISupportInitialize)grannyFileSplitContainerA).EndInit();
            grannyFileSplitContainerA.ResumeLayout(false);
            grannyFileTabContainer.Panel1.ResumeLayout(false);
            grannyFileTabContainer.Panel2.ResumeLayout(false);
            ((ISupportInitialize)grannyFileTabContainer).EndInit();
            grannyFileTabContainer.ResumeLayout(false);
            materialButtonsPanel.ResumeLayout(false);
            animationsTabControl.ResumeLayout(false);
            grannyAnimsTabPage.ResumeLayout(false);
            otherActionsTabPage.ResumeLayout(false);
            otherActionsTabPage.PerformLayout();
            selectModelTabPage.ResumeLayout(false);
            panel1.ResumeLayout(false);
            panel1.PerformLayout();
            fileInfoGroupBox.ResumeLayout(false);
            ResumeLayout(false);
		}

    }
}