﻿using Firaxis.Granny;
using System.IO;
using NexusBuddy.GrannyWrappers;
using System.Globalization;

namespace NexusBuddy.FileOps
{
    class MetadataWriter
    {
        public static void WriteGeoAnimFile (IGrannyFile file, int currentModelIndex, string className)
        {
            string fgxFilename = Path.GetFileName(file.Filename);
            string filenameNoExt = Path.GetFileNameWithoutExtension(file.Filename);
            string directory = Path.GetDirectoryName(file.Filename);
            
            string numberFormat = "f6";

            bool isAnimation = false;
            string instanceName = "Geometry";
            string fileExtension = ".geo";

            if (file.Animations.Count > 0 && file.Models.Count == 0)
            {
                isAnimation = true;
                instanceName = "Animation";
                fileExtension = ".anm";
            }

            string geoFilename = filenameNoExt + fileExtension;

            StreamWriter outputWriter = new StreamWriter(new FileStream(directory + "\\" + geoFilename, FileMode.Create));

            outputWriter.WriteLine("<?xml version=\"1.0\" encoding=\"UTF - 8\" ?>");
            outputWriter.WriteLine("<AssetObjects::" + instanceName + "Instance>");

            //m_CookParams
            outputWriter.WriteLine("<m_CookParams>");
            outputWriter.WriteLine("<m_Values/>");
            outputWriter.WriteLine("</m_CookParams>");

            //m_Version
            outputWriter.WriteLine("<m_Version>");
            outputWriter.WriteLine("<major>4</major>");
            outputWriter.WriteLine("<minor>0</minor>");
            outputWriter.WriteLine("<build>253</build>");
            outputWriter.WriteLine("<revision>867</revision>");
            outputWriter.WriteLine("</m_Version>");

            if (isAnimation)
            {
                outputWriter.WriteLine("<m_fDuration>" + file.Animations[0].Duration.ToString(numberFormat, CultureInfo.InvariantCulture) + "</m_fDuration>");
            }
            else
            {
                //m_Meshes
                outputWriter.WriteLine("<m_Meshes>");
                foreach (IGrannyMesh mesh in file.Meshes)
                {
                    GrannyMeshWrapper meshWrapper = new GrannyMeshWrapper(mesh);

                    outputWriter.WriteLine("<Element>");
                    outputWriter.WriteLine("<m_Name text=\"" + mesh.Name + "\"/>");

                    outputWriter.WriteLine("<m_Groups>");
                    foreach (IGrannyMaterial material in mesh.MaterialBindings)
                    {
                        outputWriter.WriteLine("<Element>");
                        outputWriter.WriteLine("<m_Name text=\"" + material.Name + "\"/>");
                        outputWriter.WriteLine("<m_nFirstPrim>0</m_nFirstPrim>");
                        outputWriter.WriteLine("<m_nPrims>" + meshWrapper.getGroup0TriCount() + "</m_nPrims>");
                        outputWriter.WriteLine("</Element>");
                    }
                    outputWriter.WriteLine("</m_Groups>");

                    outputWriter.WriteLine("<m_nBoundBoneCount>" + mesh.BoneBindings.Count + "</m_nBoundBoneCount>");
                    outputWriter.WriteLine("<m_nPrimitiveCount>" + meshWrapper.getGroup0TriCount() + "</m_nPrimitiveCount>");
                    outputWriter.WriteLine("<m_nVertexCount>" + mesh.VertexCount + "</m_nVertexCount>");

                    outputWriter.WriteLine("</Element>");
                }
                outputWriter.WriteLine("</m_Meshes>");

                //m_Bones
                outputWriter.WriteLine("<m_Bones>");
                foreach (IGrannyBone bone in file.Models[currentModelIndex].Skeleton.Bones)
                {
                    outputWriter.WriteLine("<Element text=\"" + bone.Name + "\"/>");
                }
                outputWriter.WriteLine("</m_Bones>");
                outputWriter.WriteLine("<m_ModelName text=\"" + file.Models[currentModelIndex].Name + "\"/>");
            }

            outputWriter.WriteLine("<m_SourceFilePath text=\"\"/>");
            outputWriter.WriteLine("<m_SourceObjectName text=\"\"/>");
            outputWriter.WriteLine("<m_ImportedTime>0</m_ImportedTime>");
            outputWriter.WriteLine("<m_ExportedTime>0</m_ExportedTime>");

            outputWriter.WriteLine("<m_ClassName text=\"" + className + "\"/>");

            //m_DataFiles
            outputWriter.WriteLine("<m_DataFiles>");
            outputWriter.WriteLine("<Element>");
            outputWriter.WriteLine("<m_ID text=\"GR2\"/>");
            outputWriter.WriteLine("<m_RelativePath text=\""+ fgxFilename + "\"/>");
            outputWriter.WriteLine("</Element>");
            outputWriter.WriteLine("</m_DataFiles>");

            outputWriter.WriteLine("<m_Name text=\"" + filenameNoExt + "\"/>");
            outputWriter.WriteLine("<m_Description text=\"" + filenameNoExt + "\"/>");

            outputWriter.WriteLine("<m_Tags>");
            outputWriter.WriteLine("<Element text=\""+ className + "\"/>");
            outputWriter.WriteLine("</m_Tags>");

            outputWriter.WriteLine("<m_Groups/>");
            outputWriter.WriteLine("</AssetObjects::" + instanceName + "Instance>");

            outputWriter.Close();
        }

        public static void WriteGeometrySet(IGrannyFile file)
        {
            string filenameNoExt = Path.GetFileNameWithoutExtension(file.Filename);
            string directory = Path.GetDirectoryName(file.Filename);
            string fileExtension = "_GeometrySet.xml";

            string geoFilename = filenameNoExt + fileExtension;

            StreamWriter outputWriter = new StreamWriter(new FileStream(directory + "\\" + geoFilename, FileMode.Create));

            outputWriter.WriteLine("<m_GeometrySet>");
            outputWriter.WriteLine("<m_ModelInstances>");

            foreach (IGrannyModel model in file.Models)
            {
                outputWriter.WriteLine("<Element>");
                outputWriter.WriteLine("<m_Name text=\"" + model.Name + "\"/>");
                outputWriter.WriteLine("<m_GeoName text=\"" + filenameNoExt + "\"/>");

                outputWriter.WriteLine("<m_GroupStates>"); // meshes start

                foreach (IGrannyMesh mesh in model.MeshBindings) { 

                    outputWriter.WriteLine("<Element>");

                    outputWriter.WriteLine("<m_Values>"); //materials start
                    outputWriter.WriteLine("<m_Values>");

                    outputWriter.WriteLine("<Element class=\"AssetObjects::ObjectValue\">");
                    outputWriter.WriteLine("<m_ObjectName text=\"" + mesh.MaterialBindings[0].Name + "\"/>");
                    outputWriter.WriteLine("<m_eObjectType>MATERIAL</m_eObjectType>");
                    outputWriter.WriteLine("<m_ParamName text=\"Material\"/>");
                    outputWriter.WriteLine("</Element>");

                    outputWriter.WriteLine("</m_Values>");
                    outputWriter.WriteLine("</m_Values>"); //materials end

                    outputWriter.WriteLine("<m_GroupName text=\"" + mesh.MaterialBindings[0].Name + "\"/>");

                    outputWriter.WriteLine("<m_MeshName text=\"" + mesh.Name + "\"/>");

                    outputWriter.WriteLine("<m_StateName text=\"Default\"/>");

                    outputWriter.WriteLine("</Element>");
                }

                outputWriter.WriteLine("</m_GroupStates>"); //meshes end

                outputWriter.WriteLine("</Element>");

            }
            outputWriter.WriteLine("</m_ModelInstances>");
            outputWriter.WriteLine("</m_GeometrySet>");

            outputWriter.Close();
        }
    }
}
